function [GG1,GG2,BB1,BB2,Ire_0,Iim_0,Node,indx_n,Phase_Nodes,N_ph_VR] = Admittance_Build_LPF(BaseMVA,BasekV,Source,branches,conf,loads,caps,regs)
% This function returns the admittance matrix for the three-phase linear
% power flow (3LPF) analysis.
% -------------------------------------------------------------------------
% 
%                  Copyright (C) 2015  Hamed Ahmadi
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% -------------------------------------------------------------------------

% Check the number of input arguments 
if nargin ~= 8
    error('Not Enough/Too Many Inputs!')
end

Nb = length(branches(:,1));% number of branches

Fr = branches(:,1);
To = branches(:,2);
% Find all the existing nodes in the system
dummy1 = [Fr;To];
[~,idx]=unique(dummy1,'stable');
dummy1 =  dummy1(idx);
% Find the source node
a_ind = find(strcmp(dummy1,Source),1);
dummy1(a_ind) = [];dummy1 = [Source;dummy1];% brings source node to the top

Nn = length(dummy1);% Number of total nodes including the source
if ~isempty(regs)
    N_VR = length(regs(:,1));% number of voltage regulators
    a_VR = zeros(N_VR,1);
    for k = 1:N_VR
        a_VR(k) = find(strcmp(branches(:,4),regs{k,3}),1);
    end
else
    N_VR = 0;
    a_VR = [];
end
N_ph_VR = 0;% Calculating the total number of phases for VRs
if N_VR>0
    for k = 1:N_VR
        N_ph_VR = N_ph_VR + length(regs{k,4});
    end
end
Y = cell(Nn+N_VR,Nn+N_VR);% initialize augmented Ybus, assuming every line section is three-phase
Bsh = repmat({zeros(3)},Nn,1);
% Adding shunt capacitors to the shunt elements
if ~isempty(caps)
    N_C = length(caps(:,1));%Number of shunt capacitors
else
    N_C = 0;
end

if N_C>0
    for k = 1:N_C
        Bsh{strcmp(dummy1(:,1),caps{k,1}),1} = 1i*(BaseMVA/1000)*diag([caps{k,2},caps{k,3},caps{k,4}]);
    end
end

S_loop = 1:Nb;S_loop(a_VR) = [];
for k = S_loop
    k1 = branches{k,4};
    %         Feet to Mile conversion, determining the branch impedance and
    %         equivalent shunt admittance in per-unit
    Z = 0.000189394*cell2mat(branches(k,3))*(BaseMVA/BasekV^2)*(conf.(k1).('R') + 1i*conf.(k1).('X'));
    B = 1i*1e-6*0.000189394*cell2mat(branches(k,3))*(BasekV^2/BaseMVA).*conf.(k1).('B');
    % 1 foot = 0.000189394 miles;
    idx1 = find(strcmp(dummy1, Fr(k)));% Find the corresponding node in dummy1
    idx2 = find(strcmp(dummy1, To(k)));
    switch conf.(k1).('config')
        case {'ABC','ACB','BCA','BAC','CAB','CBA'}
            Y{idx1,idx2} = inv(Z);
            Bsh{idx1} = Bsh{idx1} + B./2;Bsh{idx2} = Bsh{idx2} + B./2;
        case {'AB','BA'}
            Z(3,:) = [];Z(:,3) = [];B(3,:) = [];B(:,3) = [];
            y_dum = inv(Z);
            Y{idx1,idx2} = [y_dum,[0;0];[0,0,0]];
            Bsh{idx1} = Bsh{idx1} + [B/2,[0;0];[0,0,0]];Bsh{idx2} = Bsh{idx2} + [B/2,[0;0];[0,0,0]];
        case {'AC','CA'}
            Z(2,:) = [];Z(:,2) = [];B(2,:) = [];B(:,2) = [];
            y_dum = inv(Z);
            Y{idx1,idx2} = [y_dum(1,1),0,y_dum(1,2);0,0,0;y_dum(2,1),0,y_dum(2,2)];
            Bsh{idx1} = Bsh{idx1} + [B(1,1)/2,0,B(1,2)/2;0,0,0;B(2,1)/2,0,B(2,2)/2];
            Bsh{idx2} = Bsh{idx2} + [B(1,1)/2,0,B(1,2)/2;0,0,0;B(2,1)/2,0,B(2,2)/2];
        case {'BC','CB'}
            Z(1,:) = [];Z(:,1) = [];B(1,:) = [];B(:,1) = [];
            y_dum = inv(Z);
            Y{idx1,idx2} = [[0,0,0];[0;0],y_dum];
            Bsh{idx1} = Bsh{idx1} + [[0,0,0];[0;0],B/2];
            Bsh{idx2} = Bsh{idx2} + [[0,0,0];[0;0],B/2];
        case 'A'
            Z = Z(1,1);B = B(1,1);
            Y{idx1,idx2} = [1/Z,0,0;0,0,0;0,0,0];
            Bsh{idx1} = Bsh{idx1} + [B/2,0,0;0,0,0;0,0,0];
            Bsh{idx2} = Bsh{idx2} + [B/2,0,0;0,0,0;0,0,0];
        case 'B'
            Z = Z(2,2);B = B(2,2);
            Y{idx1,idx2} = [0,0,0;0,1/Z,0;0,0,0];
            Bsh{idx1} = Bsh{idx1} + [0,0,0;0,B/2,0;0,0,0];
            Bsh{idx2} = Bsh{idx2} + [0,0,0;0,B/2,0;0,0,0];
        case 'C'
            Z = Z(3,3);B = B(3,3);
            Y{idx1,idx2} = [0,0,0;0,0,0;0,0,1/Z];
            Bsh{idx1} = Bsh{idx1} + [0,0,0;0,0,0;0,0,B/2];
            Bsh{idx2} = Bsh{idx2} + [0,0,0;0,0,0;0,0,B/2];
    end
    Y{idx2,idx1} = Y{idx1,idx2};
end
%% Calculate the portion of load for constant impedance, current, and power
[Ka,Kb,Kc,Kab,Kbc,Kca] = LPF_param();
P_p = zeros(3*Nn,3);
Q_p = zeros(3*Nn,3);
Ire_a = zeros(Nn,3);Ire_b = zeros(Nn,3);Ire_c = zeros(Nn,3);
Iim_a = zeros(Nn,3);Iim_b = zeros(Nn,3);Iim_c = zeros(Nn,3);
Ire_ab = zeros(Nn,3);Ire_bc = zeros(Nn,3);Ire_ca = zeros(Nn,3);
Iim_ab = zeros(Nn,3);Iim_bc = zeros(Nn,3);Iim_ca = zeros(Nn,3);
G1_aug = cell(Nn,1);B1_aug = cell(Nn,1);
G2_aug = cell(Nn,1);B2_aug = cell(Nn,1);

for k = 1:Nn
    ind_l = strcmp(loads(:,1),dummy1(k));% Find corresponding node for each load
    if any(ind_l)
        P1 = cell2mat(loads(ind_l,3:14));
        P1(1:6) = P1(1:6)./(1000*BaseMVA);% calculate the per-unit values
        P_p(3*k-2:3*k,:) = repmat(P1(7:9)',1,3).*repmat(P1([1,3,5]),3,1);
        Q_p(3*k-2:3*k,:) = repmat(P1(10:12)',1,3).*repmat(P1([2,4,6]),3,1);
    end
    if any(ind_l)
    switch loads{ind_l,2}
        case 'Y'
            A_mult_re = [P_p(3*k-2,1)  Q_p(3*k-2,1) P_p(3*k,1)  Q_p(3*k,1) P_p(3*k-1,1)  Q_p(3*k-1,1)];
            A_mult_im = [-Q_p(3*k-2,1) P_p(3*k-2,1) -Q_p(3*k,1) P_p(3*k,1) -Q_p(3*k-1,1) P_p(3*k-1,1)];
            Ire_a(k,:) = A_mult_re*[[1 0 0;0 1 0];Ka'];
            Iim_a(k,:) = A_mult_im*[[1 0 0;0 1 0];Ka'];
            A_mult_re = [P_p(3*k-2,2)  Q_p(3*k-2,2) P_p(3*k,2)  Q_p(3*k,2) P_p(3*k-1,2)  Q_p(3*k-1,2)];
            A_mult_im = [-Q_p(3*k-2,2) P_p(3*k-2,2) -Q_p(3*k,2) P_p(3*k,2) -Q_p(3*k-1,2) P_p(3*k-1,2)];
            Ire_b(k,:) = A_mult_re*[[1 0 0;0 1 0];Kb'];
            Iim_b(k,:) = A_mult_im*[[1 0 0;0 1 0];Kb'];
            A_mult_re = [P_p(3*k-2,3)  Q_p(3*k-2,3) P_p(3*k,3)  Q_p(3*k,3) P_p(3*k-1,3)  Q_p(3*k-1,3)];
            A_mult_im = [-Q_p(3*k-2,3) P_p(3*k-2,3) -Q_p(3*k,3) P_p(3*k,3) -Q_p(3*k-1,3) P_p(3*k-1,3)];
            Ire_c(k,:) = A_mult_re*[[1 0 0;0 1 0];Kc'];
            Iim_c(k,:) = A_mult_im*[[1 0 0;0 1 0];Kc'];
            G1_aug{k} = diag([Ire_a(k,1),Ire_b(k,1),Ire_c(k,1)]);
            B1_aug{k} = diag([Ire_a(k,2),Ire_b(k,2),Ire_c(k,2)]);
            G2_aug{k} = diag([Iim_a(k,2),Iim_b(k,2),Iim_c(k,2)]);
            B2_aug{k} = diag([Iim_a(k,1),Iim_b(k,1),Iim_c(k,1)]);
        case 'D'
            A_mult_re = [P_p(3*k-2,1)/3  Q_p(3*k-2,1)/3 P_p(3*k,1)  Q_p(3*k,1) P_p(3*k-1,1)/sqrt(3)  Q_p(3*k-1,1)/sqrt(3)];
            A_mult_im = [-Q_p(3*k-2,1)/3 P_p(3*k-2,1)/3 -Q_p(3*k,1) P_p(3*k,1) -Q_p(3*k-1,1)/sqrt(3) P_p(3*k-1,1)/sqrt(3)];
            Ire_ab(k,:) = A_mult_re*[[1 0 0;0 1 0];Kab'];
            Iim_ab(k,:) = A_mult_im*[[1 0 0;0 1 0];Kab'];
            A_mult_re = [P_p(3*k-2,2)/3  Q_p(3*k-2,2)/3 P_p(3*k,2)  Q_p(3*k,2) P_p(3*k-1,2)/sqrt(3)  Q_p(3*k-1,2)/sqrt(3)];
            A_mult_im = [-Q_p(3*k-2,2)/3 P_p(3*k-2,2)/3 -Q_p(3*k,2) P_p(3*k,2) -Q_p(3*k-1,2)/sqrt(3) P_p(3*k-1,2)/sqrt(3)];
            Ire_bc(k,:) = A_mult_re*[[1 0 0;0 1 0];Kbc'];
            Iim_bc(k,:) = A_mult_im*[[1 0 0;0 1 0];Kbc'];
            A_mult_re = [P_p(3*k-2,3)/3  Q_p(3*k-2,3)/3 P_p(3*k,3)  Q_p(3*k,3) P_p(3*k-1,3)/sqrt(3)  Q_p(3*k-1,3)/sqrt(3)];
            A_mult_im = [-Q_p(3*k-2,3)/3 P_p(3*k-2,3)/3 -Q_p(3*k,3) P_p(3*k,3) -Q_p(3*k-1,3)/sqrt(3) P_p(3*k-1,3)/sqrt(3)];
            Ire_ca(k,:) = A_mult_re*[[1 0 0;0 1 0];Kca'];
            Iim_ca(k,:) = A_mult_im*[[1 0 0;0 1 0];Kca'];
            G1_aug{k} = [Ire_ab(k,1)+Ire_ca(k,1) -Ire_ab(k,1)             -Ire_ca(k,1)
                          -Ire_ab(k,1)              Ire_ab(k,1)+Ire_bc(k,1) -Ire_bc(k,1)
                          -Ire_ca(k,1)             -Ire_bc(k,1)              Ire_bc(k,1)+Ire_ca(k,1)];
            B1_aug{k} = [Ire_ab(k,2)+Ire_ca(k,2) -Ire_ab(k,2)             -Ire_ca(k,2)
                          -Ire_ab(k,2)              Ire_ab(k,2)+Ire_bc(k,2) -Ire_bc(k,2)
                          -Ire_ca(k,2)             -Ire_bc(k,2)              Ire_bc(k,2)+Ire_ca(k,2)];
            B2_aug{k} = [Iim_ab(k,1)+Iim_ca(k,1) -Iim_ab(k,1)             -Iim_ca(k,1)
                          -Iim_ab(k,1)              Iim_ab(k,1)+Iim_bc(k,1) -Iim_bc(k,1)
                          -Iim_ca(k,1)             -Iim_bc(k,1)              Iim_bc(k,1)+Iim_ca(k,1)];
            G2_aug{k} = [Iim_ab(k,2)+Iim_ca(k,2) -Iim_ab(k,2)             -Iim_ca(k,2)
                          -Iim_ab(k,2)              Iim_ab(k,2)+Iim_bc(k,2) -Iim_bc(k,2)
                          -Iim_ca(k,2)             -Iim_bc(k,2)              Iim_bc(k,2)+Iim_ca(k,2)];          
            
    end
    end
end
Ire_0 = [Ire_a(:,3),Ire_b(:,3),Ire_c(:,3)] + [Ire_ab(:,3)-Ire_ca(:,3),Ire_bc(:,3)-Ire_ab(:,3),Ire_ca(:,3)-Ire_bc(:,3)];
Iim_0 = [Iim_a(:,3),Iim_b(:,3),Iim_c(:,3)] + [Iim_ab(:,3)-Iim_ca(:,3),Iim_bc(:,3)-Iim_ab(:,3),Iim_ca(:,3)-Iim_bc(:,3)];
% Calculate diagonal elements
for k = 1:Nn
    aa = cell2mat(Y(k,:));
    bb = zeros(3);
    for k1 = 1:length(aa(1,:))/3
        bb = bb + aa(:,3*k1-2:3*k1);
    end
    Y{k,k} = -bb-Bsh{k,1};
end
% Adding the augmented values for the voltage regulators to the admittance
% matrix
if N_VR>0
    for k = 1:N_VR
        VR_F = strcmp(dummy1(:,1),regs{k,1});
        VR_T = strcmp(dummy1(:,1),regs{k,2});
        Y{Nn+k,VR_F} = -diag(cell2mat(regs(k,15:17)));
        Y{Nn+k,VR_T} = eye(3);
        Y{VR_F,Nn+k} = Y{Nn+k,VR_F};
        Y{VR_T,Nn+k} = Y{Nn+k,VR_T};
    end
end

Nv = cell(Nn,1);
% determine the phase variables at each node
for k = 1:Nn
    con1 = branches(strcmp(branches(:,1),dummy1(k)),4);
    con2 = branches(strcmp(branches(:,2),dummy1(k)),4);
    con12 = [con1;con2];% list of all the phase configs connected to this node
    if length(con12) == 1
        Nv{k,1} = conf.(char(con12)).config;
    elseif ~isempty(con12)
        con0 = cell(length(con12),2);
        for k1 = 1:length(con12)
            con0{k1,1} = conf.(char(con12(k1))).config;
            con0{k1,2} = length(con0{k1,1});
        end
        [~,b1] = max(cell2mat(con0(:,2)));
        Nv{k,1} = con0{b1,1};        
    end
end

Phase_Nodes = Nv;% phase variables at each node
N_Vol = length(strjoin(Nv','')) - 3;% number of total unknown phase voltages (source is taken out)
% Find index of non-existing phase voltages to be removed from the Ybus
indx_n = zeros(3*Nn-N_Vol-3,1);
m1 = 1;
for k = 1:Nn
    if length(Nv{k,1}) == 2
        switch Nv{k,1}
            case {'AB','BA'}
            indx_n(m1) = 3*k;
            m1 = m1+1;
            case {'AC','CA'}
            indx_n(m1) = 3*k-1;
            m1 = m1+1;
            case {'BC','CB'}
            indx_n(m1) = 3*k-2;
            m1 = m1+1;
        end
    elseif length(Nv{k,1}) == 1
        switch Nv{k,1}
            case 'A'
            indx_n(m1) = 3*k-1;
            indx_n(m1+1) = 3*k;
            m1 = m1+2;
            case 'B'
            indx_n(m1) = 3*k-2;
            indx_n(m1+1) = 3*k;
            m1 = m1+2;
            case 'C'
            indx_n(m1) = 3*k-2;
            indx_n(m1+1) = 3*k-1;
            m1 = m1+2;
        end
    end
end
        
ind_0 = cellfun(@isempty,Y);Y(ind_0) = {zeros(3)};
ind_0 = cellfun(@isempty,G1_aug);G1_aug(ind_0) = {zeros(3)};
ind_0 = cellfun(@isempty,B1_aug);B1_aug(ind_0) = {zeros(3)};
ind_0 = cellfun(@isempty,G2_aug);G2_aug(ind_0) = {zeros(3)};
ind_0 = cellfun(@isempty,B2_aug);B2_aug(ind_0) = {zeros(3)};

YY1 = Y;YY2 = Y;
for k = 1:Nn
    YY1{k,k} = YY1{k,k} + (-G1_aug{k} + 1i*B1_aug{k});
    YY2{k,k} = YY2{k,k} + (-G2_aug{k} - 1i*B2_aug{k});
end
    
YY1 = cell2mat(YY1);
YY2 = cell2mat(YY2);
% Remove non-existing phases pertained to the VRs from augmented Ybus
if N_VR>0
    indx_VR = zeros(3*N_VR-N_ph_VR,1);
    m1 = 1;
    for k = 1:N_VR
        switch regs{k,4}
            case 'AB'
                indx_VR(m1) = 3*k;
                m1 = m1+1;
            case 'AC'
                indx_VR(m1) = 3*k-1;
                m1 = m1+1;
            case 'BC'
                indx_VR(m1) = 3*k-2;
                m1 = m1+1;
            case 'A'
                indx_VR(m1) = 3*k-1;
                indx_VR(m1+1) = 3*k;
                m1 = m1+2;
            case 'B'
                indx_VR(m1) = 3*k-2;
                indx_VR(m1+1) = 3*k;
                m1 = m1+2;
            case 'C'
                indx_VR(m1) = 3*k-2;
                indx_VR(m1+1) = 3*k-1;
                m1 = m1+2;
        end
    end
    indx_VR = indx_VR + 3*Nn;
else
    indx_VR = [];
end

% Adding the resulting elements from linearization of the left handside to
YY1([indx_n;indx_VR],:) = [];YY1(:,[indx_n;indx_VR]) = [];
YY2([indx_n;indx_VR],:) = [];YY2(:,[indx_n;indx_VR]) = [];

GG1 = -real(YY1); BB1 = imag(YY1);GG2 = -real(YY2); BB2 = -imag(YY2);
% Remove current elements that does not correspond to an existing variable
Ire_0(1,:) = [];Iim_0(1,:) = [];
Ire_0 = reshape(Ire_0',size(Ire_0,1)*size(Ire_0,2),1);
Iim_0 = reshape(Iim_0',size(Iim_0,1)*size(Iim_0,2),1);

Ire_0(indx_n-3) = [];
Iim_0(indx_n-3) = [];

Node = dummy1;


end