function [ka,kb,kc,kab,kbc,kca] = LPF_param()
% This function returns the parameters needed in the three-phase linear
% power flow (3LPF) solution. For more information, refer to the following
% publication:
% H. Ahmadi, J.R. Marti, A. von Meier, "A linear power flow formulation for
% three-phase distribution systems," submitted to the IEEE Transactions on
% Power Systems, 2015.
% % -------------------------------------------------------------------------
% 
%                  Copyright (C) 2015  Hamed Ahmadi
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% -------------------------------------------------------------------------

%% Phase A
n = 50;% Number of points for function evaluation
d = 10*pi/180;% maximum voltage angle deviation
th_max =  d;
th_min = -d;
Vmin = 0.9;% minimum voltage magnitude
Vmax = 1.1;% maximum voltage magnitude
V_m = linspace(Vmin,Vmax,n);V_m = repmat(V_m',1,n);
V_a = linspace(th_min,th_max,n);V_a = repmat(V_a,n,1);
% Converting from polar to rectangular coordinates
V_re = V_m.*cos(V_a);
V_im = V_m.*sin(V_a);
x = reshape(V_re,n*n,1);y = reshape(V_im,n*n,1);
% Form the four nonlinear functions
f(:,1) = x./(x.^2 + y.^2);
f(:,2) = y./(x.^2 + y.^2);
f(:,3) = x./sqrt(x.^2 + y.^2);
f(:,4) = y./sqrt(x.^2 + y.^2);
% The following is the result of solving a least-square minimization
% problem.
A = [
    sum(x.^2)	sum(x.*y)	sum(x)
    sum(x.*y)	sum(y.^2)   sum(y)
    sum(x)      sum(y)      n*n
    ];
ka = zeros(3,4);
for ki = 1:4
    ka(:,ki) = A\[sum(x.*f(:,ki));sum(y.*f(:,ki));sum(f(:,ki))];
end
%% Phase B
th_max = -2*pi/3 + d;
th_min = -2*pi/3 - d;
V_a = linspace(th_min,th_max,n);V_a = repmat(V_a,n,1);
V_re = V_m.*cos(V_a);
V_im = V_m.*sin(V_a);
x = reshape(V_re,n*n,1);y = reshape(V_im,n*n,1);
f(:,1) = x./(x.^2 + y.^2);
f(:,2) = y./(x.^2 + y.^2);
f(:,3) = x./sqrt(x.^2 + y.^2);
f(:,4) = y./sqrt(x.^2 + y.^2);

A = [
    sum(x.^2)	sum(x.*y)	sum(x)
    sum(x.*y)	sum(y.^2)   sum(y)
    sum(x)      sum(y)      n*n
    ];
kb = zeros(3,4);
for ki = 1:4
    kb(:,ki) = A\[sum(x.*f(:,ki));sum(y.*f(:,ki));sum(f(:,ki))];
end
%% Phase C
th_max = 2*pi/3 + d;
th_min = 2*pi/3 - d;
V_a = linspace(th_min,th_max,n);V_a = repmat(V_a,n,1);
V_re = V_m.*cos(V_a);
V_im = V_m.*sin(V_a);
x = reshape(V_re,n*n,1);y = reshape(V_im,n*n,1);
f(:,1) = x./(x.^2 + y.^2);
f(:,2) = y./(x.^2 + y.^2);
f(:,3) = x./sqrt(x.^2 + y.^2);
f(:,4) = y./sqrt(x.^2 + y.^2);

A = [
    sum(x.^2)	sum(x.*y)	sum(x)
    sum(x.*y)	sum(y.^2)   sum(y)
    sum(x)      sum(y)      n*n
    ];
kc = zeros(3,4);
for ki = 1:4
    kc(:,ki) = A\[sum(x.*f(:,ki));sum(y.*f(:,ki));sum(f(:,ki))];
end
%% Phase A-B
th_max = pi/6 + d;
th_min = pi/6 - d;
Vmin = sqrt(3)*0.9;
Vmax = sqrt(3)*1.1;
V_m = linspace(Vmin,Vmax,n);V_m = repmat(V_m',1,n);
V_a = linspace(th_min,th_max,n);V_a = repmat(V_a,n,1);
V_re = V_m.*cos(V_a);
V_im = V_m.*sin(V_a);
x = reshape(V_re,n*n,1);y = reshape(V_im,n*n,1);
f(:,1) = x./(x.^2 + y.^2);
f(:,2) = y./(x.^2 + y.^2);
f(:,3) = x./sqrt(x.^2 + y.^2);
f(:,4) = y./sqrt(x.^2 + y.^2);

A = [
    sum(x.^2)	sum(x.*y)	sum(x)
    sum(x.*y)	sum(y.^2)   sum(y)
    sum(x)      sum(y)      n*n
    ];
kab = zeros(3,4);
for ki = 1:4
    kab(:,ki) = A\[sum(x.*f(:,ki));sum(y.*f(:,ki));sum(f(:,ki))];
end
%% Phase B-C
% The maximum and minimum angle deviations changes for line-line voltages
th_max = -pi/2 + d;
th_min = -pi/2 - d;
V_a = linspace(th_min,th_max,n);V_a = repmat(V_a,n,1);
V_re = V_m.*cos(V_a);
V_im = V_m.*sin(V_a);
x = reshape(V_re,n*n,1);y = reshape(V_im,n*n,1);
f(:,1) = x./(x.^2 + y.^2);
f(:,2) = y./(x.^2 + y.^2);
f(:,3) = x./sqrt(x.^2 + y.^2);
f(:,4) = y./sqrt(x.^2 + y.^2);

A = [
    sum(x.^2)	sum(x.*y)	sum(x)
    sum(x.*y)	sum(y.^2)   sum(y)
    sum(x)      sum(y)      n*n
    ];
kbc = zeros(3,4);
for ki = 1:4
    kbc(:,ki) = A\[sum(x.*f(:,ki));sum(y.*f(:,ki));sum(f(:,ki))];
end
%% Phase C-A
th_max = 5*pi/6 + d;
th_min = 5*pi/6 - d;
V_a = linspace(th_min,th_max,n);V_a = repmat(V_a,n,1);
V_re = V_m.*cos(V_a);
V_im = V_m.*sin(V_a);
x = reshape(V_re,n*n,1);y = reshape(V_im,n*n,1);
f(:,1) = x./(x.^2 + y.^2);
f(:,2) = y./(x.^2 + y.^2);
f(:,3) = x./sqrt(x.^2 + y.^2);
f(:,4) = y./sqrt(x.^2 + y.^2);

A = [
    sum(x.^2)	sum(x.*y)	sum(x)
    sum(x.*y)	sum(y.^2)   sum(y)
    sum(x)      sum(y)      n*n
    ];
kca = zeros(3,4);
for ki = 1:4
    kca(:,ki) = A\[sum(x.*f(:,ki));sum(y.*f(:,ki));sum(f(:,ki))];
end
end