function [Ire,Iim] = Load_Current(BaseMVA,loads,Nodes,indx_n,Phase_Nodes,Vre_,Vim_)
% This function calculates the equivalent current injections from each load
% given a set of nodal voltages
% -------------------------------------------------------------------------
% 
%                  Copyright (C) 2015  Hamed Ahmadi
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% -------------------------------------------------------------------------

Nn = length(Nodes(:,1));% Total number of nodes
L_ind = length(indx_n);% Number of non-existing nodes
L_V = length(Vre_);% Number of variables
% Initialize internally-used voltages (three variables for each node)
Vre1 = zeros(L_ind+L_V,1);
Vim1 = zeros(L_ind+L_V,1);
m = 1;
% add zeros to Vre and Vim for the non-existing phase variables
for k  = 1:L_ind+L_V
    if m <= L_ind
        if k == indx_n(m)
            Vre1(k) = 0;Vim1(k) = 0;
            m = m+1;
        else
            Vre1(k) = Vre_(k-m+1);Vim1(k) = Vim_(k-m+1);
        end
    else
        Vre1(k) = Vre_(k-m+1);Vim1(k) = Vim_(k-m+1);
    end
end
% Initialize the output currents
Ire = zeros(L_V,1);
Iim = zeros(L_V,1);

for k = 1:Nn-1
    ak = strcmp(Nodes{k+1,1},loads(:,1));% Find corresponding node for each load
%     If there is no load at the current node, the injection is zero 
    if ~any(ak)
        Ire(3*k-2:3*k) = zeros(3,1);
        Iim(3*k-2:3*k) = zeros(3,1);
    else
%         Find the per-unit values for active and reactive power
        P = transpose(cell2mat(loads(ak,[3,5,7])))./(1000*BaseMVA);
        Q = transpose(cell2mat(loads(ak,[4,6,8])))./(1000*BaseMVA);
        switch loads{ak,2}% Based on the load configuration (Y,D) and voltage dependence (PQ, Z, I)
            case 'Y-PQ'
                switch Phase_Nodes{k+1,1}
                    case 'ABC'
                        Ire(3*k-2:3*k) = (P.*Vre1(3*k-2:3*k)+Q.*Vim1(3*k-2:3*k))./(Vre1(3*k-2:3*k).^2 + Vim1(3*k-2:3*k).^2);
                        Iim(3*k-2:3*k) = (P.*Vim1(3*k-2:3*k)-Q.*Vre1(3*k-2:3*k))./(Vre1(3*k-2:3*k).^2 + Vim1(3*k-2:3*k).^2);
                    case 'AB'
                        Ire(3*k-2:3*k-1) = (P(1:2).*Vre1(3*k-2:3*k-1)+Q(1:2).*Vim1(3*k-2:3*k-1))./(Vre1(3*k-2:3*k-1).^2 + Vim1(3*k-2:3*k-1).^2);
                        Iim(3*k-2:3*k-1) = (P(1:2).*Vim1(3*k-2:3*k-1)-Q(1:2).*Vre1(3*k-2:3*k-1))./(Vre1(3*k-2:3*k-1).^2 + Vim1(3*k-2:3*k-1).^2);
                        Ire(3*k) = 0;Iim(3*k) = 0;
                    case 'AC'
                        Ire([3*k-2 3*k]) = (P([1 3]).*Vre1([3*k-2 3*k])+Q([1 3]).*Vim1([3*k-2 3*k]))./(Vre1([3*k-2 3*k]).^2 + Vim1([3*k-2 3*k]).^2);
                        Iim([3*k-2 3*k]) = (P([1 3]).*Vim1([3*k-2 3*k])-Q([1 3]).*Vre1([3*k-2 3*k]))./(Vre1([3*k-2 3*k]).^2 + Vim1([3*k-2 3*k]).^2);
                        Ire(3*k-1) = 0;Iim(3*k-1) = 0;
                    case 'BC'
                        Ire(3*k-1:3*k) = (P(2:3).*Vre1(3*k-1:3*k)+Q(2:3).*Vim1(3*k-1:3*k))./(Vre1(3*k-1:3*k).^2 + Vim1(3*k-1:3*k).^2);
                        Iim(3*k-1:3*k) = (P(2:3).*Vim1(3*k-1:3*k)-Q(2:3).*Vre1(3*k-1:3*k))./(Vre1(3*k-1:3*k).^2 + Vim1(3*k-1:3*k).^2);
                        Ire(3*k) = 0;Iim(3*k) = 0;
                    case 'A'
                        Ire(3*k-2) = (P(1).*Vre1(3*k-2)+Q(1).*Vim1(3*k-2))./(Vre1(3*k-2).^2 + Vim1(3*k-2).^2);
                        Iim(3*k-2) = (P(1).*Vim1(3*k-2)-Q(1).*Vre1(3*k-2))./(Vre1(3*k-2).^2 + Vim1(3*k-2).^2);
                        Ire(3*k-1:3*k) = 0;Iim(3*k-1:3*k) = 0;
                    case 'B'
                        Ire(3*k-1) = (P(2).*Vre1(3*k-1)+Q(2).*Vim1(3*k-1))./(Vre1(3*k-1).^2 + Vim1(3*k-1).^2);
                        Iim(3*k-1) = (P(2).*Vim1(3*k-1)-Q(2).*Vre1(3*k-1))./(Vre1(3*k-1).^2 + Vim1(3*k-1).^2);
                        Ire([3*k-2 3*k]) = 0;Iim([3*k-2 3*k]) = 0;
                    case 'C'
                        Ire(3*k) = (P(3).*Vre1(3*k)+Q(3).*Vim1(3*k))./(Vre1(3*k).^2 + Vim1(3*k).^2);
                        Iim(3*k) = (P(3).*Vim1(3*k)-Q(3).*Vre1(3*k))./(Vre1(3*k).^2 + Vim1(3*k).^2);
                        Ire(3*k-2:3*k-1) = 0;Iim(3*k-2:3*k-1) = 0;
                end
            case 'Y-Z'
                switch Phase_Nodes{k+1,1}
                    case 'ABC'
                        Ire(3*k-2:3*k) = P.*Vre1(3*k-2:3*k)+Q.*Vim1(3*k-2:3*k);
                        Iim(3*k-2:3*k) = P.*Vim1(3*k-2:3*k)-Q.*Vre1(3*k-2:3*k);
                    case 'AB'
                        Ire(3*k-2:3*k-1) = P(1:2).*Vre1(3*k-2:3*k-1)+Q(1:2).*Vim1(3*k-2:3*k-1);
                        Iim(3*k-2:3*k-1) = P(1:2).*Vim1(3*k-2:3*k-1)-Q(1:2).*Vre1(3*k-2:3*k-1);
                        Ire(3*k) = 0;Iim(3*k) = 0;
                    case 'AC'
                        Ire([3*k-2 3*k]) = P([1 3]).*Vre1([3*k-2 3*k])+Q([1 3]).*Vim1([3*k-2 3*k]);
                        Iim([3*k-2 3*k]) = P([1 3]).*Vim1([3*k-2 3*k])-Q([1 3]).*Vre1([3*k-2 3*k]);
                        Ire(3*k-1) = 0;Iim(3*k-1) = 0;
                    case 'BC'
                        Ire(3*k-1:3*k) = P(2:3).*Vre1(3*k-1:3*k)+Q(2:3).*Vim1(3*k-1:3*k);
                        Iim(3*k-1:3*k) = P(2:3).*Vim1(3*k-1:3*k)-Q(2:3).*Vre1(3*k-1:3*k);
                        Ire(k) = 0;Iim(k) = 0;
                    case 'A'
                        Ire(3*k-2) = P(1).*Vre1(3*k-2)+Q(1).*Vim1(3*k-2);
                        Iim(3*k-2) = P(1).*Vim1(3*k-2)-Q(1).*Vre1(3*k-2);
                        Ire(3*k-1:3*k) = 0;Iim(3*k-1:3*k) = 0;
                    case 'B'
                        Ire(3*k-1) = P(2).*Vre1(3*k-1)+Q(2).*Vim1(3*k-1);
                        Iim(3*k-1) = P(2).*Vim1(3*k-1)-Q(2).*Vre1(3*k-1);
                        Ire([3*k-2 3*k]) = 0;Iim([3*k-2 3*k]) = 0;
                    case 'C'
                        Ire(3*k) = P(3).*Vre1(3*k)+Q(3).*Vim1(3*k);
                        Iim(3*k) = P(3).*Vim1(3*k)-Q(3).*Vre1(3*k);
                        Ire(3*k-2:3*k-1) = 0;Iim(3*k-2:3*k-1) = 0;
                end
            case 'Y-I'
                switch Phase_Nodes{k+1,1}
                    case 'ABC'
                        Ire(3*k-2:3*k) = (P.*Vre1(3*k-2:3*k)+Q.*Vim1(3*k-2:3*k))./sqrt(Vre1(3*k-2:3*k).^2 + Vim1(3*k-2:3*k).^2);
                        Iim(3*k-2:3*k) = (P.*Vim1(3*k-2:3*k)-Q.*Vre1(3*k-2:3*k))./sqrt(Vre1(3*k-2:3*k).^2 + Vim1(3*k-2:3*k).^2);
                    case 'AB'
                        Ire(3*k-2:3*k-1) = (P(1:2).*Vre1(3*k-2:3*k-1)+Q(1:2).*Vim1(3*k-2:3*k-1))./sqrt(Vre1(3*k-2:3*k-1).^2 + Vim1(3*k-2:3*k-1).^2);
                        Iim(3*k-2:3*k-1) = (P(1:2).*Vim1(3*k-2:3*k-1)-Q(1:2).*Vre1(3*k-2:3*k-1))./sqrt(Vre1(3*k-2:3*k-1).^2 + Vim1(3*k-2:3*k-1).^2);
                        Ire(3*k) = 0;Iim(3*k) = 0;
                    case 'AC'
                        Ire([3*k-2 3*k]) = (P([1 3]).*Vre1([3*k-2 3*k])+Q([1 3]).*Vim1([3*k-2 3*k]))./sqrt(Vre1([3*k-2 3*k]).^2 + Vim1([3*k-2 3*k]).^2);
                        Iim([3*k-2 3*k]) = (P([1 3]).*Vim1([3*k-2 3*k])-Q([1 3]).*Vre1([3*k-2 3*k]))./sqrt(Vre1([3*k-2 3*k]).^2 + Vim1([3*k-2 3*k]).^2);
                        Ire(3*k-1) = 0;Iim(3*k-1) = 0;
                    case 'BC'
                        Ire(3*k-1:3*k) = (P(2:3).*Vre1(3*k-1:3*k)+Q(2:3).*Vim1(3*k-1:3*k))./sqrt(Vre1(3*k-1:3*k).^2 + Vim1(3*k-1:3*k).^2);
                        Iim(3*k-1:3*k) = (P(2:3).*Vim1(3*k-1:3*k)-Q(2:3).*Vre1(3*k-1:3*k))./sqrt(Vre1(3*k-1:3*k).^2 + Vim1(3*k-1:3*k).^2);
                        Ire(k) = 0;Iim(k) = 0;
                    case 'A'
                        Ire(3*k-2) = (P(1).*Vre1(3*k-2)+Q(1).*Vim1(3*k-2))./sqrt(Vre1(3*k-2).^2 + Vim1(3*k-2).^2);
                        Iim(3*k-2) = (P(1).*Vim1(3*k-2)-Q(1).*Vre1(3*k-2))./sqrt(Vre1(3*k-2).^2 + Vim1(3*k-2).^2);
                        Ire(3*k-1:3*k) = 0;Iim(3*k-1:3*k) = 0;
                    case 'B'
                        Ire(3*k-1) = (P(2).*Vre1(3*k-1)+Q(2).*Vim1(3*k-1))./sqrt(Vre1(3*k-1).^2 + Vim1(3*k-1).^2);
                        Iim(3*k-1) = (P(2).*Vim1(3*k-1)-Q(2).*Vre1(3*k-1))./sqrt(Vre1(3*k-1).^2 + Vim1(3*k-1).^2);
                        Ire([3*k-2 3*k]) = 0;Iim([3*k-2 3*k]) = 0;
                    case 'C'
                        Ire(3*k) = (P(3).*Vre1(3*k)+Q(3).*Vim1(3*k))./sqrt(Vre1(3*k).^2 + Vim1(3*k).^2);
                        Iim(3*k) = (P(3).*Vim1(3*k)-Q(3).*Vre1(3*k))./sqrt(Vre1(3*k).^2 + Vim1(3*k).^2);
                        Ire(3*k-2:3*k-1) = 0;Iim(3*k-2:3*k-1) = 0;
                end
            case 'D-PQ'
                A = [1 0 -1;-1 1 0;0 -1 1];
                Vre_p2p = A'*Vre1(3*k-2:3*k);
                Vim_p2p = A'*Vim1(3*k-2:3*k); 
                Ire_p2p = (P.*Vre_p2p + Q.*Vim_p2p)./(Vre_p2p.^2 + Vim_p2p.^2);
                Iim_p2p = (P.*Vim_p2p - Q.*Vre_p2p)./(Vre_p2p.^2 + Vim_p2p.^2);
                Ire(3*k-2:3*k,1) = A*Ire_p2p;
                Iim(3*k-2:3*k,1) = A*Iim_p2p;
            case 'D-Z'
                A = [1 0 -1;-1 1 0;0 -1 1];
                Vre_p2p = A'*Vre1(3*k-2:3*k);
                Vim_p2p = A'*Vim1(3*k-2:3*k); 
                Ire_p2p = (P.*Vre_p2p + Q.*Vim_p2p)./3;
                Iim_p2p = (P.*Vim_p2p - Q.*Vre_p2p)./3;
                Ire(3*k-2:3*k,1) = A*Ire_p2p;
                Iim(3*k-2:3*k,1) = A*Iim_p2p;               
            case 'D-I'
                A = [1 0 -1;-1 1 0;0 -1 1];
                Vre_p2p = A'*Vre1(3*k-2:3*k);
                Vim_p2p = A'*Vim1(3*k-2:3*k); 
                Ire_p2p = (P.*Vre_p2p + Q.*Vim_p2p)./(sqrt(3)*sqrt(Vre_p2p.^2 + Vim_p2p.^2));
                Iim_p2p = (P.*Vim_p2p - Q.*Vre_p2p)./(sqrt(3)*sqrt(Vre_p2p.^2 + Vim_p2p.^2));
                Ire(3*k-2:3*k,1) = A*Ire_p2p;
                Iim(3*k-2:3*k,1) = A*Iim_p2p;
        end
    end
end
% Remove non-existing elements
Ire(indx_n) = [];Iim(indx_n) = [];

end


