% This m-file calls the routines for specific test systems to load the
% corresponding power flow data and then calls the two power flow routines,
% namely the current injection method (CIM) and the three-phase linear
% power flow (3LPF).

% -------------------------------------------------------------------------
% 
%                  Copyright (C) 2015  Hamed Ahmadi
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% -------------------------------------------------------------------------

clc
clear
% Choose a test system:
% 1: A hypothetical 5-node system
% 2: The IEEE 13-node system
% 3: The IEEE 37-node system
% 4: The IEEE 123-node system
% 5: The IEEE 8500-node system
test = 2;
switch test
    case 1
        conf = Node5_Build();
        [BaseMVA,BasekV,Source,branches,loads,caps,regs] = Node5_Data();
    case 2
        conf = IEEE13Node_Build();
        [BaseMVA,BasekV,Source,branches,loads,caps,regs] = IEEE13Node_Data();
    case 3
        conf = IEEE37Node_Build();
        [BaseMVA,BasekV,Source,branches,loads,caps,regs] = IEEE37Node_Data();
    case 4
        conf = IEEE123Node_Build();
        [BaseMVA,BasekV,Source,branches,loads,caps,regs] = IEEE123Node_Data();
    case 5
        conf = IEEE8500Node_Build();
        [BaseMVA,BasekV,Source,branches,loads,caps,regs] = IEEE8500Node_Data();
end
% Choose the substation voltage magnitude
V0 = 1.00;
% Run the current injection method and the 3LPF
results_CIM = CIM_3Ph(BaseMVA,BasekV,Source,branches,loads,caps,regs,conf,V0);
results_LPF = LPF_3Ph(BaseMVA,BasekV,Source,branches,loads,caps,regs,conf,V0);
% Find empty cells in the results corresponding to non-existing phases
i_empty = cellfun(@isempty,results_CIM);
% replace the empty cells with an arbitrary value, e.g., 100
results_CIM(i_empty) = {100};
results_LPF(i_empty) = {100};
% Convert from cell to matrix
Vabc_CIM = cell2mat(results_CIM(2:end,2:4));
Vabc_LPF = cell2mat(results_LPF(2:end,2:4));
% Find non-empty cells
i_empty = not(i_empty(2:end,2:4));
% Calculate the relative error
Error = 100*abs(Vabc_CIM(i_empty) - Vabc_LPF(i_empty))./Vabc_CIM(i_empty);
% Show the histogram of errors
[a,b] = hist(Error);a = 100*a./sum(a);
bar(b,a);xlabel('Relative Error(%)');ylabel('Density (%)');
title('Error histogram of 3LPF considering all phases');
